import { UtilsProvider } from '@provider/utils.provider'
import AnimatedBackground from '@shared/components/background/background.view'
import i18next from 'i18next'
import React from 'react'
import ReactDOM from 'react-dom/client'
import { ErrorBoundary } from "react-error-boundary"
import { I18nextProvider } from 'react-i18next'
import { SkeletonTheme } from 'react-loading-skeleton'
import { Provider } from 'react-redux'
import {
    RouterProvider
} from 'react-router-dom'
import { ToastContainer } from 'react-toastify'
import { PersistGate } from 'redux-persist/integration/react'
import './i18n/config'
import './index.scss'
import { router } from './routes'
import './shared/icons/icons'
import { persistor, store } from './stores'

window.env = window.location.hostname === 'localhost' ? 'development' : 'production';

ReactDOM.createRoot(document.getElementById("root")).render(
    <React.StrictMode>
        <AnimatedBackground>
            <ErrorBoundary fallback={<div>Something went wrong</div>}>
                <SkeletonTheme height="2em" >
                    <Provider store={store}>
                        <PersistGate loading={null} persistor={persistor}>
                            <I18nextProvider i18n={i18next}>
                                <UtilsProvider>
                                    <ToastContainer />
                                    <RouterProvider router={router} />
                                </UtilsProvider>
                            </I18nextProvider>
                        </PersistGate>
                    </Provider>
                </SkeletonTheme>
            </ErrorBoundary>
        </AnimatedBackground>
    </React.StrictMode>
);

